@@ -1,17 +1,20 @@
+from __future__ import absolute_import
+
 from distutils import version
 import os
 import warnings
 
 try:
     import matplotlib
-    import matplotlib.pyplot as mpl
-    import mpl_toolkits.axes_grid.parasite_axes as mpltk
 except ImportError:
     raise Exception("matplotlib 0.99.0 or later is required for APLpy")
 
 if version.LooseVersion(matplotlib.__version__) < version.LooseVersion('0.99.0'):
     raise Exception("matplotlib 0.99.0 or later is required for APLpy")
 
+import matplotlib.pyplot as mpl
+import mpl_toolkits.axes_grid.parasite_axes as mpltk
+
 try:
     import pyfits
 except ImportError:
@@ -27,33 +30,55 @@ try:
 except ImportError:
     raise Exception("numpy is required for APLpy")
 
-from matplotlib.patches import Circle, Rectangle, Ellipse, Polygon
-from matplotlib.collections import PatchCollection
+from matplotlib.patches import Circle, Rectangle, Ellipse, Polygon, FancyArrow
+from matplotlib.collections import PatchCollection, LineCollection
 
-import contour_util
+try:
+    import montage
+    if not hasattr(montage, 'reproject_hdu'):
+        raise
+    montage_installed = True
+except:
+    montage_installed = False
 
-import convolve_util
+try:
+    import Image
+    pil_installed = True
+except:
+    pil_installed = False
 
-import montage
-import image_util
-import header
-import wcs_util
-import slicer
+try:
+    from pyavm import AVM
+    avm_installed = True
+except:
+    avm_installed = False
 
-from layers import Layers
-from grid import Grid
-from ticks import Ticks
-from labels import TickLabels
-from axis_labels import AxisLabels
-from overlays import Beam, ScaleBar
-from regions import Regions
-from colorbar import Colorbar
-from normalize import APLpyNormalize
-from frame import Frame
+if montage_installed:
+    if version.LooseVersion(montage.__version__) < version.LooseVersion('0.9.2'):
+        warnings.warn("Python-montage installation is not recent enough (version 0.9.2 or later is required). Disabling Montage-related functionality.")
+        montage_installed = False
 
-from decorators import auto_refresh, fixdocstring
+import aplpy.contour_util as contour_util
+import aplpy.convolve_util as convolve_util
+import aplpy.image_util as image_util
+import aplpy.header as header
+import aplpy.wcs_util as wcs_util
+import aplpy.slicer as slicer
 
-from deprecated import Deprecated
+from aplpy.layers import Layers
+from aplpy.grid import Grid
+from aplpy.ticks import Ticks
+from aplpy.labels import TickLabels
+from aplpy.axis_labels import AxisLabels
+from aplpy.overlays import Beam, ScaleBar
+from aplpy.regions import Regions
+from aplpy.colorbar import Colorbar
+from aplpy.normalize import APLpyNormalize
+from aplpy.frame import Frame
+
+from aplpy.decorators import auto_refresh, fixdocstring
+
+from aplpy.deprecated import Deprecated
 
 
 class FITSFigure(Layers, Regions, Deprecated):
@@ -67,9 +92,11 @@ class FITSFigure(Layers, Regions, Deprecated):
 
         Required arguments:
 
-            *data*: [ string | pyfits PrimaryHDU | pyfits ImageHDU ]
-                Either the filename of the FITS file to open, or a pyfits
-                PrimaryHDU or ImageHDU instance.
+            *data*: [ string | pyfits.PrimaryHDU | pyfits.ImageHDU | pywcs.WCS ]
+                Either the filename of the FITS file to open, a pyfits
+                PrimaryHDU or ImageHDU object, or a pywcs WCS object. It is
+                also possible to specify the filename of an RGB image tagged
+                with AVM meta-data.
 
         Optional Keyword Arguments:
 
@@ -128,11 +155,43 @@ class FITSFigure(Layers, Regions, Deprecated):
         if not 'figsize' in kwargs:
             kwargs['figsize'] = (10, 9)
 
-        if north and not montage._installed():
-            raise Exception("Montage needs to be installed and in the $PATH in order to use the north= option")
+        if type(data) is str and data.split('.')[-1].lower() in ['png', 'jpg', 'tif']:
 
-        self._hdu, self._wcs = self._get_hdu(data, hdu, north, \
-            convention=convention, slices=slices)
+            if not pil_installed:
+                raise Exception("The Python Imaging Library (PIL) is required to read in RGB images")
+
+            if not avm_installed:
+                raise Exception("PyAVM is required to read in AVM meta-data from RGB images")
+
+            # Remember image filename
+            self._rgb_image = data
+
+            # Find image size
+            nx, ny = Image.open(data).size
+
+            # Now convert AVM information to WCS
+            data = AVM(data).to_wcs()
+
+            # Need to scale CDELT values sometimes the AVM meta-data is only really valid for the full-resolution image
+            data.wcs.cdelt = [data.wcs.cdelt[0] * data.naxis1 / float(nx), data.wcs.cdelt[1] * data.naxis2 / float(ny)]
+            data.wcs.crpix = [data.wcs.crpix[0] / data.naxis1 * float(nx), data.wcs.crpix[1] / data.naxis2 * float(ny)]
+
+            # Update the NAXIS values with the true dimensions of the RGB image
+            data.naxis1 = nx
+            data.naxis2 = ny
+
+        if isinstance(data, pywcs.WCS):
+
+            self._hdu, self._wcs = pyfits.ImageHDU(data=np.zeros((data.naxis2, data.naxis1), dtype=float), header=data.to_header()), data
+            if downsample:
+                warnings.warn("downsample argument is ignored if data passed is a WCS object")
+                downsample = False
+            if north:
+                warnings.warn("north argument is ignored if data passed is a WCS object")
+                north = False
+        else:
+            self._hdu, self._wcs = self._get_hdu(data, hdu, north, \
+                convention=convention, slices=slices)
 
         # Downsample if requested
         if downsample:
@@ -216,10 +275,29 @@ class FITSFigure(Layers, Regions, Deprecated):
 
             # Read in FITS file
             try:
-                hdu = pyfits.open(filename)[hdu]
+                hdulist = pyfits.open(filename)
             except:
                 raise Exception("An error occured while reading the FITS file")
 
+            # Check whether the HDU specified contains any data, otherwise
+            # cycle through all HDUs to find one that contains valid image data
+            if hdulist[hdu].data is None:
+                found = False
+                for alt_hdu in range(len(hdulist)):
+                    if isinstance(hdulist[alt_hdu], pyfits.PrimaryHDU) or \
+                       isinstance(hdulist[alt_hdu], pyfits.ImageHDU):
+                        if hdulist[alt_hdu].data is not None:
+                            warnings.warn("hdu=%i does not contain any data, using hdu=%i instead" % (hdu, alt_hdu))
+                            hdu = hdulist[alt_hdu]
+                            found = True
+                            break
+                if not found:
+                    raise Exception("FITS file does not contain any image data")
+
+            else:
+                hdu = hdulist[hdu]
+
+
         elif isinstance(data, pyfits.PrimaryHDU) or isinstance(data, pyfits.ImageHDU):
 
             hdu = data
@@ -237,7 +315,9 @@ class FITSFigure(Layers, Regions, Deprecated):
 
         # Reproject to face north if requested
         if north:
-            hdu = montage.reproject_north(hdu)
+            if not montage_installed:
+                raise Exception("Both the Montage command-line tools and the Python-montage module are required to use the north= argument")
+            hdu = montage.reproject_hdu(hdu, north_aligned=True)
 
         # Check header
         hdu.header = header.check(hdu.header, convention=convention)
@@ -317,9 +397,10 @@ class FITSFigure(Layers, Regions, Deprecated):
         self._ax1.set_ylim(ypix-dy_pix, ypix+dy_pix)
 
     @auto_refresh
-    def show_grayscale(self, vmin=None, vmid=None, vmax=None, \
-                            pmin=0.25, pmax=99.75, \
-                            stretch='linear', exponent=2, invert='default', smooth=None, kernel='gauss'):
+    def show_grayscale(self, vmin=None, vmid=None, vmax=None,
+                            pmin=0.25, pmax=99.75,
+                            stretch='linear', exponent=2, invert='default',
+                            smooth=None, kernel='gauss', interpolation='nearest'):
         '''
         Show a grayscale image of the FITS file
 
@@ -347,8 +428,9 @@ class FITSFigure(Layers, Regions, Deprecated):
                 The stretch function to use
 
             *vmid*: [ None | float ]
-                Mid-pixel value used for the log and arcsinh stretches. If
-                set to None, this is set to a sensible value.
+                Baseline value used for the log and arcsinh stretches. If
+                set to None, this is set to zero for log stretches and to
+                vmin - (vmax - vmin) / 30. for arcsinh stretches
 
             *exponent*: [ float ]
                 If stretch is set to 'power', this is the exponent to use
@@ -368,6 +450,14 @@ class FITSFigure(Layers, Regions, Deprecated):
                 Default kernel used for smoothing is 'gauss'. The user can
                 specify if they would prefer 'gauss', 'box', or a custom
                 kernel. All kernels are normalized to ensure flux retention.
+
+            *interpolation*: [ string ]
+                The type of interpolation to use for the image. The default is
+                'nearest'. Other options include 'none' (no interpolation,
+                meaning that if exported to a postscript file, the grayscale
+                will be output at native resolution irrespective of the dpi
+                setting), 'bilinear', 'bicubic', and many more (see the
+                matplotlib documentation for imshow).
         '''
 
         if invert=='default':
@@ -378,7 +468,7 @@ class FITSFigure(Layers, Regions, Deprecated):
         else:
             cmap = 'gray'
 
-        self.show_colorscale(vmin=vmin, vmid=vmid, vmax=vmax, stretch=stretch, exponent=exponent, cmap=cmap, pmin=pmin, pmax=pmax, smooth=smooth, kernel=kernel)
+        self.show_colorscale(vmin=vmin, vmid=vmid, vmax=vmax, stretch=stretch, exponent=exponent, cmap=cmap, pmin=pmin, pmax=pmax, smooth=smooth, kernel=kernel, interpolation=interpolation)
 
     @auto_refresh
     def hide_grayscale(self, *args, **kwargs):
@@ -387,7 +477,7 @@ class FITSFigure(Layers, Regions, Deprecated):
     @auto_refresh
     def show_colorscale(self, vmin=None, vmid=None, vmax=None, \
                              pmin=0.25, pmax=99.75,
-                             stretch='linear', exponent=2, cmap='default', smooth=None, kernel='gauss'):
+                             stretch='linear', exponent=2, cmap='default', smooth=None, kernel='gauss', interpolation='nearest'):
         '''
         Show a colorscale image of the FITS file
 
@@ -415,8 +505,9 @@ class FITSFigure(Layers, Regions, Deprecated):
                 The stretch function to use
 
             *vmid*: [ None | float ]
-                Mid-pixel value used for the log and arcsinh stretches. If
-                set to None, this is set to a sensible value.
+                Baseline value used for the log and arcsinh stretches. If
+                set to None, this is set to zero for log stretches and to
+                vmin - (vmax - vmin) / 30. for arcsinh stretches
 
             *exponent*: [ float ]
                 If stretch is set to 'power', this is the exponent to use
@@ -434,6 +525,14 @@ class FITSFigure(Layers, Regions, Deprecated):
                 Default kernel used for smoothing is 'gauss'. The user can
                 specify if they would prefer 'gauss', 'box', or a custom
                 kernel. All kernels are normalized to ensure flux retention.
+
+            *interpolation*: [ string ]
+                The type of interpolation to use for the image. The default is
+                'nearest'. Other options include 'none' (no interpolation,
+                meaning that if exported to a postscript file, the colorscale
+                will be output at native resolution irrespective of the dpi
+                setting), 'bilinear', 'bicubic', and many more (see the
+                matplotlib documentation for imshow).
         '''
 
         if cmap=='default':
@@ -443,7 +542,7 @@ class FITSFigure(Layers, Regions, Deprecated):
         max_auto = np.equal(vmax, None)
 
         # The set of available functions
-        cmap = mpl.cm.get_cmap(cmap, 1000)
+        cmap = mpl.cm.get_cmap(cmap)
 
         if min_auto:
             vmin = self._auto_v(pmin)
@@ -457,16 +556,12 @@ class FITSFigure(Layers, Regions, Deprecated):
 
         # Adjust vmin/vmax if auto
         if min_auto:
-            if stretch <> 'linear':
-                warnings.warn("Auto-scaling for non-linear stretches may produce slightly different results compared to APLpy 0.9.4")
-            else:
+            if stretch == 'linear':
                 vmin = -0.1 * (vmax - vmin) + vmin
             print "Auto-setting vmin to %10.3e" % vmin
 
         if max_auto:
-            if stretch <> 'linear':
-                warnings.warn("Auto-scaling for non-linear stretches may produce slightly different results compared to APLpy 0.9.4")
-            else:
+            if stretch == 'linear':
                 vmax = 0.1 * (vmax - vmin) + vmax
             print "Auto-setting vmax to %10.3e" % vmax
 
@@ -479,9 +574,10 @@ class FITSFigure(Layers, Regions, Deprecated):
             self.image.set_norm(normalizer)
             self.image.set_cmap(cmap=cmap)
             self.image.origin='lower'
-            self.image.set_data(convolve_util.convolve(self._hdu.data,smooth=smooth,kernel=kernel))
+            self.image.set_interpolation(interpolation)
+            self.image.set_data(convolve_util.convolve(self._hdu.data, smooth=smooth, kernel=kernel))
         else:
-            self.image = self._ax1.imshow(convolve_util.convolve(self._hdu.data,smooth=smooth,kernel=kernel), cmap=cmap, interpolation='nearest', origin='lower', extent=self._extent, norm=normalizer)
+            self.image = self._ax1.imshow(convolve_util.convolve(self._hdu.data, smooth=smooth, kernel=kernel), cmap=cmap, interpolation=interpolation, origin='lower', extent=self._extent, norm=normalizer)
 
         xmin, xmax = self._ax1.get_xbound()
         if xmin == 0.0:
@@ -512,29 +608,46 @@ class FITSFigure(Layers, Regions, Deprecated):
         self.image.set_cmap(cm)
 
     @auto_refresh
-    def show_rgb(self, filename, interpolation='nearest', flip=False):
+    def show_rgb(self, filename=None, interpolation='nearest', vertical_flip=False, horizontal_flip=False, flip=False):
         '''
         Show a 3-color image instead of the FITS file data
 
-        Required Arguments:
-
-            *filename*
-                The 3-color image should have exactly the same dimensions as the FITS file, and
-                will be shown with exactly the same projection.
-
         Optional Arguments:
 
-            *flip*: [ True | False ]
-                Whether to vertically flip the RGB image in case it is the
-                wrong way around.
+            *filename*
+                The 3-color image should have exactly the same dimensions
+                as the FITS file, and will be shown with exactly the same
+                projection. If FITSFigure was initialized with an
+                AVM-tagged RGB image, the filename is not needed here.
+
+            *vertical_flip*: [ True | False ]
+                Whether to vertically flip the RGB image
+
+            *horizontal_flip*: [ True | False ]
+                Whether to horizontally flip the RGB image
         '''
 
-        img = mpl.imread(filename)
+        if not pil_installed:
+            raise Exception("The Python Imaging Library (PIL) is required to read in RGB images")
 
         if flip:
-            img = np.flipud(img)
+            warnings.warn("Note that show_rgb should now correctly flip RGB images, so the flip= argument is now deprecated. If you still need to flip an image vertically or horizontally, you can use the vertical_flip= and horizontal_flip arguments instead.")
 
-        self.image = self._ax1.imshow(img, extent=self._extent, interpolation=interpolation, origin='upper')
+        if filename is None:
+            if hasattr(self, '_rgb_image'):
+                image = Image.open(self._rgb_image)
+            else:
+                raise Exception("Need to specify the filename of an RGB image")
+        else:
+            image = Image.open(filename)
+
+        if vertical_flip:
+            image = image.transpose(Image.FLIP_TOP_BOTTOM)
+
+        if horizontal_flip:
+            image = image.transpose(Image.FLIP_LEFT_RIGHT)
+
+        self.image = self._ax1.imshow(image, extent=self._extent, interpolation=interpolation, origin='lower')
 
     @auto_refresh
     def show_contour(self, data, hdu=0, layer=None, levels=5, filled=False, cmap=None, colors=None, returnlevels=False, convention=None, slices=[], smooth=None, kernel='gauss', **kwargs):
@@ -614,14 +727,14 @@ class FITSFigure(Layers, Regions, Deprecated):
             self.remove_layer(layer, raise_exception=False)
 
         if cmap:
-            cmap = mpl.cm.get_cmap(cmap, 1000)
+            cmap = mpl.cm.get_cmap(cmap)
         elif not colors:
-            cmap = mpl.cm.get_cmap('jet', 1000)
+            cmap = mpl.cm.get_cmap('jet')
 
         hdu_contour, wcs_contour = self._get_hdu(data, hdu, False, \
             convention=convention, slices=slices)
 
-        image_contour = convolve_util.convolve(hdu_contour.data,smooth=smooth,kernel=kernel)
+        image_contour = convolve_util.convolve(hdu_contour.data, smooth=smooth, kernel=kernel)
         extent_contour = (0.5, wcs_contour.naxis1+0.5, 0.5, wcs_contour.naxis2+0.5)
 
         if type(levels) == int:
@@ -703,7 +816,7 @@ class FITSFigure(Layers, Regions, Deprecated):
     # Show circles. Different from markers as this method allows more definitions
     # for the circles.
     @auto_refresh
-    def show_circles(self, xw, yw, radius, layer=False, **kwargs):
+    def show_circles(self, xw, yw, radius, layer=False, zorder=None, **kwargs):
         '''
         Overlay circles on the current plot.
 
@@ -761,7 +874,10 @@ class FITSFigure(Layers, Regions, Deprecated):
         for i in range(len(xp)):
             patches.append(Circle((xp[i], yp[i]), radius=rp[i], **kwargs))
 
-        c = self._ax1.add_collection(PatchCollection(patches, match_original=True))
+        p = PatchCollection(patches, match_original=True)
+        if zorder is not None:
+            p.zorder = zorder
+        c = self._ax1.add_collection(p)
 
         if layer:
             circle_set_name = layer
@@ -772,7 +888,7 @@ class FITSFigure(Layers, Regions, Deprecated):
         self._layers[circle_set_name] = c
 
     @auto_refresh
-    def show_ellipses(self, xw, yw, width, height, layer=False, **kwargs):
+    def show_ellipses(self, xw, yw, width, height, angle=0, layer=False, zorder=None, **kwargs):
         '''
        Overlay ellipses on the current plot.
 
@@ -824,6 +940,11 @@ class FITSFigure(Layers, Regions, Deprecated):
         else:
             width = np.array(width)
 
+        if np.isscalar(angle):
+            angle = np.repeat(angle, len(xw))
+        else:
+            angle = np.array(angle)
+
         if np.isscalar(height):
             height = np.repeat(height, len(xw))
         else:
@@ -838,12 +959,16 @@ class FITSFigure(Layers, Regions, Deprecated):
         xp, yp = wcs_util.world2pix(self._wcs, xw, yw)
         wp = 3600.0*width/wcs_util.arcperpix(self._wcs)
         hp = 3600.0*height/wcs_util.arcperpix(self._wcs)
+        ap = angle
 
         patches = []
         for i in range(len(xp)):
-            patches.append(Ellipse((xp[i], yp[i]), width=wp[i], height=hp[i], **kwargs))
+            patches.append(Ellipse((xp[i], yp[i]), width=wp[i], height=hp[i], angle=ap[i],**kwargs))
 
-        c = self._ax1.add_collection(PatchCollection(patches, match_original=True))
+        p = PatchCollection(patches, match_original=True)
+        if zorder is not None:
+            p.zorder = zorder
+        c = self._ax1.add_collection(p)
 
         if layer:
             ellipse_set_name = layer
@@ -854,7 +979,7 @@ class FITSFigure(Layers, Regions, Deprecated):
         self._layers[ellipse_set_name] = c
 
     @auto_refresh
-    def show_rectangles(self, xw, yw, width, height, layer=False, **kwargs):
+    def show_rectangles(self, xw, yw, width, height, layer=False, zorder=None, **kwargs):
         '''
        Overlay rectangles on the current plot.
 
@@ -923,7 +1048,10 @@ class FITSFigure(Layers, Regions, Deprecated):
         for i in range(len(xp)):
             patches.append(Rectangle((xp[i], yp[i]), width=wp[i], height=hp[i], **kwargs))
 
-        c = self._ax1.add_collection(PatchCollection(patches, match_original=True))
+        p = PatchCollection(patches, match_original=True)
+        if zorder is not None:
+            p.zorder = zorder
+        c = self._ax1.add_collection(p)
 
         if layer:
             rectangle_set_name = layer
@@ -934,15 +1062,154 @@ class FITSFigure(Layers, Regions, Deprecated):
         self._layers[rectangle_set_name] = c
 
     @auto_refresh
-    def show_polygons(self, polygon_list, layer=False, **kwargs):
+    def show_lines(self, line_list, layer=False, zorder=None, **kwargs):
+        '''
+       Overlay lines on the current plot.
+
+       Required arguments:
+
+           *line_list*: [ list ]
+                A list of one or more 2xN numpy arrays which contain
+                the [x, y] positions of the vertices in world coordinates.
+
+       Optional Keyword Arguments:
+
+           *layer*: [ string ]
+               The name of the line(s) layer. This is useful for giving
+               custom names to layers (instead of line_set_n) and for
+               replacing existing layers.
+
+       Additional keyword arguments (such as color, offsets, cmap,
+       or linewidth) can be used to control the appearance of the
+       lines, which is an instances of the matplotlib LineCollection class.
+       For more information on available arguments, see `LineCollection
+       <http://matplotlib.sourceforge.net/api/collections_api.html?
+       highlight=linecollection#matplotlib.collections.LineCollection>`_.
+       '''
+
+
+        if not 'color' in kwargs:
+            kwargs.setdefault('color', 'none')
+
+        if layer:
+            self.remove_layer(layer, raise_exception=False)
+
+        lines = []
+
+        for line in line_list:
+            xp, yp = wcs_util.world2pix(self._wcs, line[0, :],line[1, :])
+            lines.append(np.column_stack((xp, yp)))
+
+        l = LineCollection(lines, **kwargs)
+        if zorder is not None:
+            l.zorder = zorder
+        c = self._ax1.add_collection(l)
+
+        if layer:
+            line_set_name = layer
+        else:
+            self._linelist_counter += 1
+            line_set_name = 'line_set_'+str(self._linelist_counter)
+
+        self._layers[line_set_name] = c
+
+    @auto_refresh
+    def show_arrows(self, x, y, dx, dy, width='auto', head_width='auto',
+                    head_length='auto', layer=False, zorder=None, **kwargs):
+        '''
+       Overlay arrows on the current plot.
+
+       Required arguments:
+
+           *x, y, dx, dy*: [ float | list | array ]
+               Origin and displacement of the arrows in world coordinates.
+               These can either be scalars to plot a single arrow, or lists or
+               arrays to plot multiple arrows.
+
+       Optional Keyword Arguments:
+
+           *width*: [ float ]
+               The width of the arrow body, in pixels (default: 2% of the
+               arrow length)
+
+           *head_width*: [ float ]
+               The width of the arrow head, in pixels (default: 5% of the
+               arrow length)
+
+           *head_length*: [ float ]
+               The length of the arrow head, in pixels (default: 5% of the
+               arrow length)
+
+           *layer*: [ string ]
+               The name of the arrow(s) layer. This is useful for giving
+               custom names to layers (instead of line_set_n) and for
+               replacing existing layers.
+
+       Additional keyword arguments (such as color, offsets, cmap,
+       or linewidth) can be used to control the appearance of the
+       arrows, which is an instances of the matplotlib FancyArrow class.
+       For more information on available arguments, see `FancyArrow
+       <http://matplotlib.sourceforge.net/api/artist_api.html?
+       highlight=arrow#matplotlib.patches.FancyArrow>`_.
+       '''
+
+        if 'length_includes_head' not in kwargs:
+            kwargs['length_includes_head'] = True
+
+        if layer:
+            self.remove_layer(layer, raise_exception=False)
+
+        arrows = []
+
+        if np.isscalar(x):
+            x, y, dx, dy = [x], [y], [dx], [dy]
+
+        for i in range(len(x)):
+
+            xp1, yp1 = wcs_util.world2pix(self._wcs, x[i], y[i])
+            xp2, yp2 = wcs_util.world2pix(self._wcs, x[i]+dx[i], y[i]+dy[i])
+
+            if width == 'auto':
+                kwargs['width'] = 0.02 * np.sqrt((xp2 - xp1) ** 2 + (yp2 - yp1) ** 2)
+            else:
+                kwargs['width'] = width
+
+            if head_width == 'auto':
+                kwargs['head_width'] = 0.1 * np.sqrt((xp2 - xp1) ** 2 + (yp2 - yp1) ** 2)
+            else:
+                kwargs['head_width'] = head_width
+
+            if head_length == 'auto':
+                kwargs['head_length'] = 0.1 * np.sqrt((xp2 - xp1) ** 2 + (yp2 - yp1) ** 2)
+            else:
+                kwargs['head_length'] = head_length
+
+            arrows.append(FancyArrow(xp1, yp1, xp2 - xp1, yp2 - yp1, **kwargs))
+
+        p = PatchCollection(arrows, match_original=True)
+        if zorder is not None:
+            p.zorder = zorder
+        c = self._ax1.add_collection(p)
+
+        if layer:
+            line_set_name = layer
+        else:
+            self._linelist_counter += 1
+            line_set_name = 'arrow_set_'+str(self._linelist_counter)
+
+        self._layers[line_set_name] = c
+
+    @auto_refresh
+    def show_polygons(self, polygon_list, layer=False, zorder=None, **kwargs):
         '''
         Overlay polygons on the current plot.
 
         Required arguments:
 
-            *polygon_list*: [ list ]
-                A list of one or more 2xN numpy arrays which contain
-                the [x,y] positions of the vertices in world coordinates.
+            *polygon_list*: [ list or tuple ]
+                A list of one or more 2xN or Nx2 Numpy arrays which contain
+                the [x, y] positions of the vertices in world coordinates.
+                Note that N should be greater than 2.
 
         Optional Keyword Arguments:
 
@@ -965,18 +1232,35 @@ class FITSFigure(Layers, Regions, Deprecated):
         if layer:
             self.remove_layer(layer, raise_exception=False)
 
+        if type(polygon_list) not in [list, tuple]:
+            raise Exception("polygon_list should be a list or tuple of Numpy arrays")
+
         pix_polygon_list = []
-        for i in range(len(polygon_list)):
-            xw = polygon_list[i][:,0]
-            yw = polygon_list[i][:,1]
+        for polygon in polygon_list:
+
+            if type(polygon) is not np.ndarray:
+                raise Exception("Polygon should be given as a Numpy array")
+
+            if polygon.shape[0] == 2 and polygon.shape[1] > 2:
+                xw = polygon[0, :]
+                yw = polygon[1, :]
+            elif polygon.shape[0] > 2 and polygon.shape[1] == 2:
+                xw = polygon[:, 0]
+                yw = polygon[:, 1]
+            else:
+                raise Exception("Polygon should have dimensions 2xN or Nx2 with N>2")
+
             xp, yp = wcs_util.world2pix(self._wcs, xw, yw)
-            pix_polygon_list.append(np.column_stack((xp,yp)))
+            pix_polygon_list.append(np.column_stack((xp, yp)))
 
         patches = []
         for i in range(len(pix_polygon_list)):
             patches.append(Polygon(pix_polygon_list[i], **kwargs))
 
-        c = self._ax1.add_collection(PatchCollection(patches, match_original=True))
+        p = PatchCollection(patches, match_original=True)
+        if zorder is not None:
+            p.zorder = zorder
+        c = self._ax1.add_collection(p)
 
         if layer:
             poly_set_name = layer
@@ -1022,6 +1306,15 @@ class FITSFigure(Layers, Regions, Deprecated):
         if fontproperties:
             kwargs['fontproperties'] = fontproperties
 
+        if not np.isscalar(x):
+            raise Exception("x should be a single value")
+
+        if not np.isscalar(y):
+            raise Exception("y should be a single value")
+
+        if not np.isscalar(text):
+            raise Exception("text should be a single value")
+
         if relative:
             l = self._ax1.text(x, y, text, color=color,
                                family=family, style=style, variant=variant,
@@ -1118,7 +1411,7 @@ class FITSFigure(Layers, Regions, Deprecated):
         artists = []
         if adjust_bbox:
             for artist in self._layers.values():
-                if hasattr(artist, 'get_window_extent'):
+                if isinstance(artist, matplotlib.text.Text):
                     artists.append(artist)
             self._figure.savefig(filename, dpi=dpi, transparent=transparent, bbox_inches='tight', bbox_extra_artists=artists)
         else:
@@ -1126,10 +1419,10 @@ class FITSFigure(Layers, Regions, Deprecated):
 
     def _initialize_view(self):
 
-        self._ax1.xaxis.set_view_interval(+0.5, self._wcs.naxis1+0.5)
-        self._ax1.yaxis.set_view_interval(+0.5, self._wcs.naxis2+0.5)
-        self._ax2.xaxis.set_view_interval(+0.5, self._wcs.naxis1+0.5)
-        self._ax2.yaxis.set_view_interval(+0.5, self._wcs.naxis2+0.5)
+        self._ax1.xaxis.set_view_interval(+0.5, self._wcs.naxis1+0.5, ignore=True)
+        self._ax1.yaxis.set_view_interval(+0.5, self._wcs.naxis2+0.5, ignore=True)
+        self._ax2.xaxis.set_view_interval(+0.5, self._wcs.naxis1+0.5, ignore=True)
+        self._ax2.yaxis.set_view_interval(+0.5, self._wcs.naxis2+0.5, ignore=True)
 
         # set the image extent to FITS pixel coordinates
         self._extent = (0.5, self._wcs.naxis1+0.5, 0.5, self._wcs.naxis2+0.5)
@@ -1161,7 +1454,7 @@ class FITSFigure(Layers, Regions, Deprecated):
             self._figure.apl_grayscale_invert_default = False
             self._figure.apl_colorscale_cmap_default = 'jet'
             if self.image:
-                self.image.set_cmap(cmap=mpl.cm.get_cmap('jet', 1000))
+                self.image.set_cmap(cmap=mpl.cm.get_cmap('jet'))
         elif theme=='publication':
             self.frame.set_color('black')
             self.frame.set_linewidth(0.5)
@@ -1170,7 +1463,7 @@ class FITSFigure(Layers, Regions, Deprecated):
             self._figure.apl_grayscale_invert_default = True
             self._figure.apl_colorscale_cmap_default = 'gist_heat'
             if self.image:
-                self.image.set_cmap(cmap=mpl.cm.get_cmap('gist_yarg', 1000))
+                self.image.set_cmap(cmap=mpl.cm.get_cmap('gist_yarg'))
 
     def world2pixel(self, xw, yw):
         '''
@@ -1253,23 +1546,52 @@ class FITSFigure(Layers, Regions, Deprecated):
             >>> f.beam.set_color('white')
             >>> f.beam.set_hatch('+')
             >>> ...
+
+        If more than one beam is added, the beam object becomes a list. In
+        this case, to control the aspect of one of the beams, you will need tp
+        specify the beam index::
+
+            >>> ...
+            >>> f.beam[2].set_hatch('/')
+            >>> ...
         '''
+
+        # Initalize the beam and set parameters
+        b = Beam(self)
+        b.show(*args, **kwargs)
+
         if hasattr(self, 'beam'):
-            raise Exception("Beam already exists")
-        try:
-            self.beam = Beam(self)
-            self.beam.show(*args, **kwargs)
-        except:
-            del self.beam
-            raise
+            if type(self.beam) is list:
+                self.beam.append(b)
+            else:
+                self.beam = [self.beam, b]
+        else:
+            self.beam = b
 
     @auto_refresh
-    def remove_beam(self):
+    def remove_beam(self, beam_index=None):
         '''
-        Removes the beam from the current figure
+        Removes the beam from the current figure. If more than one beam is
+        present, the index of the beam should be specified using beam_index=
         '''
-        self.beam._remove()
-        del self.beam
+
+        if type(self.beam) is list:
+
+            if beam_index is None:
+                raise Exception("More than one beam present - use beam_index= to specify which one to remove")
+            else:
+                b = self.beam.pop(beam_index)
+                b._remove()
+                del b
+
+            # If only one beam is present, remove containing list
+            if len(self.beam) == 1:
+                self.beam = self.beam[0]
+
+        else:
+
+            self.beam._remove()
+            del self.beam
 
     @auto_refresh
     def add_scalebar(self, *args, **kwargs):
@@ -1335,3 +1657,9 @@ class FITSFigure(Layers, Regions, Deprecated):
         '''
         self.colorbar._remove()
         del self.colorbar
+
+    def close(self):
+        '''
+        Close the figure and free up the memory
+        '''
+        mpl.close(self._figure)
