@@ -1,10 +1,11 @@
 from decorators import auto_refresh
 
 class Frame(object):
-    
+
     @auto_refresh
     def __init__(self, parent):
         self._ax1 = parent._ax1
+        self._ax2 = parent._ax2
         self._figure = parent._figure
 
     @auto_refresh
@@ -19,6 +20,8 @@ class Frame(object):
         '''
         for key in self._ax1.spines:
             self._ax1.spines[key].set_linewidth(linewidth)
+        for key in self._ax2.spines:
+            self._ax2.spines[key].set_linewidth(linewidth)
 
     @auto_refresh
     def set_color(self, color):
@@ -31,4 +34,6 @@ class Frame(object):
                 The color to use for the frame.
         '''
         for key in self._ax1.spines:
-            self._ax1.spines[key].set_edgecolor(color)
\ No newline at end of file
+            self._ax1.spines[key].set_edgecolor(color)
+        for key in self._ax2.spines:
+            self._ax2.spines[key].set_edgecolor(color)
